from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_page
from drf_spectacular.utils import extend_schema
from rest_framework.generics import RetrieveAPIView

from arkindex.documents.models import Corpus, Element
from arkindex.documents.serializers.iiif import ElementAnnotationListSerializer, FolderManifestSerializer


class FolderManifest(RetrieveAPIView):
    """
    Retrieve an IIIF manifest for a folder element.
    """
    serializer_class = FolderManifestSerializer

    def get_queryset(self):
        return Element.objects.filter(
            corpus__in=Corpus.objects.readable(self.request.user),
            type__folder=True,
        )

    @method_decorator(cache_page(3600))
    @extend_schema(
        operation_id="RetrieveFolderManifest",
        responses={200: {"type": "object"}},
        tags=["iiif"],
    )
    def get(self, *args, **kwargs):
        return super().get(*args, **kwargs)


class ElementAnnotationList(RetrieveAPIView):
    """
    Retrieve an IIIF annotation list for transcriptions on elements with zones
    """

    serializer_class = ElementAnnotationListSerializer

    def get_queryset(self):
        return Element.objects.filter(
            corpus__in=Corpus.objects.readable(self.request.user),
            image_id__isnull=False,
            polygon__isnull=False,
        )

    @method_decorator(cache_page(3600))
    @extend_schema(
        operation_id="RetrieveElementAnnotationList",
        responses={200: {"type": "object"}},
        tags=["iiif"],
    )
    def get(self, *args, **kwargs):
        return super().get(*args, **kwargs)
